@@ -1,6 +1,6 @@
 /*
 htop - htop.c
-(C) 2004,2005 Hisham H. Muhammad
+(C) 2004-2006 Hisham H. Muhammad
 Released under the GNU GPL, see the COPYING file
 in the source distribution for its full text.
 */
@@ -14,7 +14,7 @@ in the source distribution for its full text.
 
 #include "ProcessList.h"
 #include "CRT.h"
-#include "ListBox.h"
+#include "Panel.h"
 #include "UsersTable.h"
 #include "SignalItem.h"
 #include "RichString.h"
@@ -22,8 +22,8 @@ in the source distribution for its full text.
 #include "ScreenManager.h"
 #include "FunctionBar.h"
 #include "ListItem.h"
-#include "CategoriesListBox.h"
-#include "SignalsListBox.h"
+#include "CategoriesPanel.h"
+#include "SignalsPanel.h"
 #include "TraceScreen.h"
 
 #include "config.h"
@@ -33,41 +33,49 @@ in the source distribution for its full text.
 
 #define INCSEARCH_MAX 40
 
-/* private property */
-char htop_barCharacters[] = "|#*@$%&";
-
 void printVersionFlag() {
    clear();
-   printf("htop " VERSION " - (C) 2004,2005 Hisham Muhammad.\n");
+   printf("htop " VERSION " - (C) 2004-2006 Hisham Muhammad.\n");
    printf("Released under the GNU GPL.\n\n");
    exit(0);
 }
 
 void printHelpFlag() {
    clear();
-   printf("htop " VERSION " - (C) 2004,2005 Hisham Muhammad.\n");
+   printf("htop " VERSION " - (C) 2004-2006 Hisham Muhammad.\n");
    printf("Released under the GNU GPL.\n\n");
    printf("-d DELAY     Delay between updates, in tenths of seconds\n\n");
    printf("-u USERNAME  Show only processes of a given user\n\n");
+   printf("--sort-key COLUMN  Sort by this column (use --sort-key help for a column list)\n\n");
    printf("Press F1 inside htop for online help.\n");
    printf("See the man page for full information.\n\n");
    exit(0);
 }
 
-void showHelp() {
+void showHelp(ProcessList* pl) {
    clear();
    attrset(CRT_colors[HELP_BOLD]);
-   mvaddstr(0, 0, "htop " VERSION " - (C) 2004 Hisham Muhammad.");
+   mvaddstr(0, 0, "htop " VERSION " - (C) 2004-2006 Hisham Muhammad.");
    mvaddstr(1, 0, "Released under the GNU GPL. See 'man' page for more info.");
 
    attrset(CRT_colors[DEFAULT_COLOR]);
    mvaddstr(3, 0, "CPU usage bar: ");
    #define addattrstr(a,s) attrset(a);addstr(s)
    addattrstr(CRT_colors[BAR_BORDER], "[");
-   addattrstr(CRT_colors[CPU_NICE], "low-priority"); addstr("/");
-   addattrstr(CRT_colors[CPU_NORMAL], "normal"); addstr("/");
-   addattrstr(CRT_colors[CPU_KERNEL], "kernel");
-   addattrstr(CRT_colors[BAR_SHADOW], "      used%");
+   if (pl->expandSystemTime) {
+      addattrstr(CRT_colors[CPU_NICE], "low"); addstr("/");
+      addattrstr(CRT_colors[CPU_NORMAL], "normal"); addstr("/");
+      addattrstr(CRT_colors[CPU_KERNEL], "kernel"); addstr("/");
+      addattrstr(CRT_colors[CPU_IOWAIT], "io-wait"); addstr("/");
+      addattrstr(CRT_colors[CPU_IRQ], "irq"); addstr("/");
+      addattrstr(CRT_colors[CPU_SOFTIRQ], "soft-irq");
+      addattrstr(CRT_colors[BAR_SHADOW], " used%");
+   } else {
+      addattrstr(CRT_colors[CPU_NICE], "low-priority"); addstr("/");
+      addattrstr(CRT_colors[CPU_NORMAL], "normal"); addstr("/");
+      addattrstr(CRT_colors[CPU_KERNEL], "kernel");
+      addattrstr(CRT_colors[BAR_SHADOW], "             used%");
+   }
    addattrstr(CRT_colors[BAR_BORDER], "]");
    attrset(CRT_colors[DEFAULT_COLOR]);
    mvaddstr(4, 0, "Memory bar:    ");
@@ -75,45 +83,46 @@ void showHelp() {
    addattrstr(CRT_colors[MEMORY_USED], "used"); addstr("/");
    addattrstr(CRT_colors[MEMORY_BUFFERS], "buffers"); addstr("/");
    addattrstr(CRT_colors[MEMORY_CACHE], "cache");
-   addattrstr(CRT_colors[BAR_SHADOW], "         used/total");
+   addattrstr(CRT_colors[BAR_SHADOW], "                used/total");
    addattrstr(CRT_colors[BAR_BORDER], "]");
    attrset(CRT_colors[DEFAULT_COLOR]);
    mvaddstr(5, 0, "Swap bar:      ");
    addattrstr(CRT_colors[BAR_BORDER], "[");
    addattrstr(CRT_colors[SWAP], "used");
-   addattrstr(CRT_colors[BAR_SHADOW], "                       used/total");
+   addattrstr(CRT_colors[BAR_SHADOW], "                              used/total");
    addattrstr(CRT_colors[BAR_BORDER], "]");
    attrset(CRT_colors[DEFAULT_COLOR]);
-   mvaddstr(6,0, "Type and layout of header meters is configurable in the setup screen.");
+   mvaddstr(6,0, "Type and layout of header meters are configurable in the setup screen.");
+   mvaddstr(7, 0, "Status: R: running; S: sleeping; T: traced/stopped; Z: zombie; D: disk sleep");
 
-   mvaddstr( 8, 0, " Arrows: scroll process list             F5 t: tree view");
-   mvaddstr( 9, 0, " Digits: incremental PID search             u: show processes of a single user");
-   mvaddstr(10, 0, "   F3 /: incremental name search            H: hide/show user threads");
-   mvaddstr(11, 0, "                                            K: hide/show kernel threads");
-   mvaddstr(12, 0, "  Space: tag processes                      F: cursor follows process");
-   mvaddstr(13, 0, "      U: untag all processes");
-   mvaddstr(14, 0, "   F9 k: kill process/tagged processes      P: sort by CPU%");
-   mvaddstr(15, 0, " + [ F7: lower priority (+ nice)            M: sort by MEM%");
-   mvaddstr(16, 0, " - ] F8: higher priority (root only)        T: sort by TIME");
-   mvaddstr(17, 0, "                                         F4 I: invert sort order");
-   mvaddstr(18, 0, "   F2 S: setup                           F6 >: select sort column");
-   mvaddstr(19, 0, "   F1 h: show this help screen");
-   mvaddstr(20, 0, "  F10 q: quit                               s: trace syscalls with strace");
+   mvaddstr( 9, 0, " Arrows: scroll process list             F5 t: tree view");
+   mvaddstr(10, 0, " Digits: incremental PID search             u: show processes of a single user");
+   mvaddstr(11, 0, "   F3 /: incremental name search            H: hide/show user threads");
+   mvaddstr(12, 0, "                                            K: hide/show kernel threads");
+   mvaddstr(13, 0, "  Space: tag processes                      F: cursor follows process");
+   mvaddstr(14, 0, "      U: untag all processes");
+   mvaddstr(15, 0, "   F9 k: kill process/tagged processes      P: sort by CPU%");
+   mvaddstr(16, 0, " + [ F7: lower priority (+ nice)            M: sort by MEM%");
+   mvaddstr(17, 0, " - ] F8: higher priority (root only)        T: sort by TIME");
+   mvaddstr(18, 0, "                                         F4 I: invert sort order");
+   mvaddstr(19, 0, "   F2 S: setup                           F6 >: select sort column");
+   mvaddstr(20, 0, "   F1 h: show this help screen");
+   mvaddstr(21, 0, "  F10 q: quit                               s: trace syscalls with strace");
 
    attrset(CRT_colors[HELP_BOLD]);
-   mvaddstr( 8, 0, " Arrows"); mvaddstr( 8,40, " F5 t");
-   mvaddstr( 9, 0, " Digits"); mvaddstr( 9,40, "    u");
-   mvaddstr(10, 0, "   F3 /"); mvaddstr(10,40, "    H");
-                               mvaddstr(11,40, "    K");
-   mvaddstr(12, 0, "  Space"); mvaddstr(12,40, "    F");
-   mvaddstr(13, 0, "      U");
-   mvaddstr(14, 0, "   F9 k"); mvaddstr(14,40, "    P");
-   mvaddstr(15, 0, " + [ F7"); mvaddstr(15,40, "    M");
-   mvaddstr(16, 0, " - ] F8"); mvaddstr(16,40, "    T");
-                               mvaddstr(17,40, " F4 I");
-   mvaddstr(18, 0, "   F2 S"); mvaddstr(18,40, " F6 >");
-   mvaddstr(19, 0, "   F1 h");
-   mvaddstr(20, 0, "  F10 q"); mvaddstr(20,40, "    s");
+   mvaddstr( 9, 0, " Arrows"); mvaddstr( 9,40, " F5 t");
+   mvaddstr(10, 0, " Digits"); mvaddstr(10,40, "    u");
+   mvaddstr(11, 0, "   F3 /"); mvaddstr(11,40, "    H");
+                               mvaddstr(12,40, "    K");
+   mvaddstr(13, 0, "  Space"); mvaddstr(13,40, "    F");
+   mvaddstr(14, 0, "      U");
+   mvaddstr(15, 0, "   F9 k"); mvaddstr(15,40, "    P");
+   mvaddstr(16, 0, " + [ F7"); mvaddstr(16,40, "    M");
+   mvaddstr(17, 0, " - ] F8"); mvaddstr(17,40, "    T");
+                               mvaddstr(18,40, " F4 I");
+   mvaddstr(19, 0, "   F2 S"); mvaddstr(19,40, " F6 >");
+   mvaddstr(20, 0, "   F1 h");
+   mvaddstr(21, 0, "  F10 q"); mvaddstr(21,40, "    s");
    attrset(CRT_colors[DEFAULT_COLOR]);
 
    attrset(CRT_colors[HELP_BOLD]);
@@ -126,62 +135,62 @@ void showHelp() {
 
 static void Setup_run(Settings* settings, int headerHeight) {
    ScreenManager* scr = ScreenManager_new(0, headerHeight, 0, -1, HORIZONTAL, true);
-   CategoriesListBox* lbCategories = CategoriesListBox_new(settings, scr);
-   ScreenManager_add(scr, (ListBox*) lbCategories, NULL, 16);
-   CategoriesListBox_makeMetersPage(lbCategories);
-   ListBox* lbFocus;
+   CategoriesPanel* panelCategories = CategoriesPanel_new(settings, scr);
+   ScreenManager_add(scr, (Panel*) panelCategories, NULL, 16);
+   CategoriesPanel_makeMetersPage(panelCategories);
+   Panel* panelFocus;
    int ch;
-   ScreenManager_run(scr, &lbFocus, &ch);
+   ScreenManager_run(scr, &panelFocus, &ch);
    ScreenManager_delete(scr);
 }
 
-static bool changePriority(ListBox* lb, int delta) {
+static bool changePriority(Panel* panel, int delta) {
    bool anyTagged = false;
-   for (int i = 0; i < ListBox_getSize(lb); i++) {
-      Process* p = (Process*) ListBox_get(lb, i);
+   for (int i = 0; i < Panel_getSize(panel); i++) {
+      Process* p = (Process*) Panel_get(panel, i);
       if (p->tag) {
          Process_setPriority(p, p->nice + delta);
          anyTagged = true;
       }
    }
    if (!anyTagged) {
-      Process* p = (Process*) ListBox_getSelected(lb);
+      Process* p = (Process*) Panel_getSelected(panel);
       Process_setPriority(p, p->nice + delta);
    }
    return anyTagged;
 }
 
-static HandlerResult pickWithEnter(ListBox* lb, int ch) {
+static HandlerResult pickWithEnter(Panel* panel, int ch) {
    if (ch == 13)
       return BREAK_LOOP;
    return IGNORED;
 }
 
-static Object* pickFromList(ListBox* lb, ListBox* list, int x, int y, char** keyLabels, FunctionBar* prevBar) {
+static Object* pickFromList(Panel* panel, Panel* list, int x, int y, char** keyLabels, FunctionBar* prevBar) {
    char* fuKeys[2] = {"Enter", "Esc"};
    int fuEvents[2] = {13, 27};
-   if (!lb->eventHandler)
-      ListBox_setEventHandler(list, pickWithEnter);
+   if (!panel->eventHandler)
+      Panel_setEventHandler(list, pickWithEnter);
    ScreenManager* scr = ScreenManager_new(0, y, 0, -1, HORIZONTAL, false);
    ScreenManager_add(scr, list, FunctionBar_new(2, keyLabels, fuKeys, fuEvents), x - 1);
-   ScreenManager_add(scr, lb, NULL, -1);
-   ListBox* lbFocus;
+   ScreenManager_add(scr, panel, NULL, -1);
+   Panel* panelFocus;
    int ch;
-   ScreenManager_run(scr, &lbFocus, &ch);
+   ScreenManager_run(scr, &panelFocus, &ch);
    ScreenManager_delete(scr);
-   ListBox_move(lb, 0, y);
-   ListBox_resize(lb, COLS, LINES-y-1);
+   Panel_move(panel, 0, y);
+   Panel_resize(panel, COLS, LINES-y-1);
    FunctionBar_draw(prevBar, NULL);
-   if (lbFocus == list && ch == 13) {
-      return ListBox_getSelected(list);
+   if (panelFocus == list && ch == 13) {
+      return Panel_getSelected(list);
    }
    return NULL;
 }
 
-void addUserToList(int key, void* userCast, void* lbCast) {
+void addUserToList(int key, void* userCast, void* panelCast) {
    char* user = (char*) userCast;
-   ListBox* lb = (ListBox*) lbCast;
-   ListBox_add(lb, (Object*) ListItem_new(user, key));
+   Panel* panel = (Panel*) panelCast;
+   Panel_add(panel, (Object*) ListItem_new(user, key));
 }
 
 void setUserOnly(const char* userName, bool* userOnly, uid_t* userId) {
@@ -197,12 +206,25 @@ int main(int argc, char** argv) {
    int delay = -1;
    bool userOnly = false;
    uid_t userId = 0;
+   int sortKey = 0;
 
    if (argc > 0) {
       if (String_eq(argv[1], "--help")) {
          printHelpFlag();
       } else if (String_eq(argv[1], "--version")) {
          printVersionFlag();
+      } else if (String_eq(argv[1], "--sort-key")) {
+         if (argc < 2) printHelpFlag();
+            if (String_eq(argv[2], "help")) {
+            for (int j = 1; j < LAST_PROCESSFIELD; j++)
+               printf ("%s\n", Process_fieldNames[j]);
+            exit(0);
+         }
+         sortKey = ColumnsPanel_fieldNameToIndex(argv[2]);
+         if (sortKey == -1) {
+            fprintf(stderr, "Error: invalid column \"%s\".\n", argv[2]);
+            exit(1);
+         }
       } else if (String_eq(argv[1], "-d")) {
          if (argc < 2) printHelpFlag();
          sscanf(argv[2], "%d", &delay);
@@ -213,15 +235,20 @@ int main(int argc, char** argv) {
          setUserOnly(argv[2], &userOnly, &userId);
       }
    }
+   
+   if (access(PROCDIR, R_OK) != 0) {
+      fprintf(stderr, "Error: could not read procfs (compiled to look in %s).\n", PROCDIR);
+      exit(1);
+   }
 
-   ListBox* lb;
+   Panel* panel;
    int quit = 0;
    int refreshTimeout = 0;
    int resetRefreshTimeout = 5;
    bool doRefresh = true;
    Settings* settings;
    
-   ListBox* lbk = NULL;
+   Panel* killPanel = NULL;
 
    char incSearchBuffer[INCSEARCH_MAX];
    int incSearchIndex = 0;
@@ -235,6 +262,10 @@ int main(int argc, char** argv) {
    
    Header* header = Header_new(pl);
    settings = Settings_new(pl, header);
+   if (sortKey > 0) {
+      pl->sortKey = sortKey;
+      pl->treeView = false;
+   }
    int headerHeight = Header_calculateHeight(header);
 
    // FIXME: move delay code to settings
@@ -243,8 +274,8 @@ int main(int argc, char** argv) {
    
    CRT_init(settings->delay, settings->colorScheme);
    
-   lb = ListBox_new(0, headerHeight, COLS, LINES - headerHeight - 2, PROCESS_CLASS, false);
-   ListBox_setRichHeader(lb, ProcessList_printHeader(pl));
+   panel = Panel_new(0, headerHeight, COLS, LINES - headerHeight - 2, PROCESS_CLASS, false, NULL);
+   Panel_setRichHeader(panel, ProcessList_printHeader(pl));
    
    char* searchFunctions[3] = {"Next  ", "Exit  ", " Search: "};
    char* searchKeys[3] = {"F3", "Esc", "  "};
@@ -268,6 +299,8 @@ int main(int argc, char** argv) {
    double oldTime = 0.0;
    bool recalculate;
 
+   int ch = 0;
+   int closeTimeout = 0;
 
    while (!quit) {
       gettimeofday(&tv, NULL);
@@ -278,9 +311,9 @@ int main(int argc, char** argv) {
       if (doRefresh) {
          incSearchIndex = 0;
          incSearchBuffer[0] = 0;
-         int currPos = ListBox_getSelectedIndex(lb);
-         int currPid = 0;
-         int currScrollV = lb->scrollV;
+         int currPos = Panel_getSelectedIndex(panel);
+         unsigned int currPid = 0;
+         int currScrollV = panel->scrollV;
          if (follow)
             currPid = ProcessList_get(pl, currPos)->pid;
          if (recalculate)
@@ -289,18 +322,18 @@ int main(int argc, char** argv) {
             ProcessList_sort(pl);
             refreshTimeout = 1;
          }
-         ListBox_prune(lb);
+         Panel_prune(panel);
          int size = ProcessList_size(pl);
-         int lbi = 0;
+         int index = 0;
          for (int i = 0; i < size; i++) {
             Process* p = ProcessList_get(pl, i);
             if (!userOnly || (p->st_uid == userId)) {
-               ListBox_set(lb, lbi, (Object*)p);
-               if ((!follow && lbi == currPos) || (follow && p->pid == currPid)) {
-                  ListBox_setSelected(lb, lbi);
-                  lb->scrollV = currScrollV;
+               Panel_set(panel, index, (Object*)p);
+               if ((!follow && index == currPos) || (follow && p->pid == currPid)) {
+                  Panel_setSelected(panel, index);
+                  panel->scrollV = currScrollV;
                }
-               lbi++;
+               index++;
             }
          }
       }
@@ -308,14 +341,26 @@ int main(int argc, char** argv) {
       
       Header_draw(header);
 
-      ListBox_draw(lb, true);
-      int ch = getch();
+      Panel_draw(panel, true);
+      int prev = ch;
+      ch = getch();
+
+      if (ch == ERR) {
+         if (!incSearchMode)
+            refreshTimeout--;
+         if (prev == ch && !recalculate) {
+            closeTimeout++;
+            if (closeTimeout == 10)
+               break;
+         } else
+            closeTimeout = 0;
+         continue;
+      }
+
       if (incSearchMode) {
          doRefresh = false;
-         if (ch == ERR) {
-            continue;
-         } else if (ch == KEY_F(3)) {
-            int here = ListBox_getSelectedIndex(lb);
+         if (ch == KEY_F(3)) {
+            int here = Panel_getSelectedIndex(panel);
             int size = ProcessList_size(pl);
             int i = here+1;
             while (i != here) {
@@ -323,7 +368,7 @@ int main(int argc, char** argv) {
                   i = 0;
                Process* p = ProcessList_get(pl, i);
                if (String_contains_i(p->comm, incSearchBuffer)) {
-                  ListBox_setSelected(lb, i);
+                  Panel_setSelected(panel, i);
                   break;
                }
                i++;
@@ -348,7 +393,7 @@ int main(int argc, char** argv) {
          for (int i = 0; i < ProcessList_size(pl); i++) {
             Process* p = ProcessList_get(pl, i);
             if (String_contains_i(p->comm, incSearchBuffer)) {
-               ListBox_setSelected(lb, i);
+               Panel_setSelected(panel, i);
                found = true;
                break;
             }
@@ -361,9 +406,9 @@ int main(int argc, char** argv) {
          continue;
       }
       if (isdigit((char)ch)) {
-         int pid = ch-48 + acc;
-         for (int i = 0; i < ProcessList_size(pl) && ((Process*) ListBox_getSelected(lb))->pid != pid; i++)
-            ListBox_setSelected(lb, i);
+         unsigned int pid = ch-48 + acc;
+         for (int i = 0; i < ProcessList_size(pl) && ((Process*) Panel_getSelected(panel))->pid != pid; i++)
+            Panel_setSelected(panel, i);
          acc = pid * 10;
          if (acc > 100000)
             acc = 0;
@@ -376,8 +421,8 @@ int main(int argc, char** argv) {
          MEVENT mevent;
          int ok = getmouse(&mevent);
          if (ok == OK) {
-            if (mevent.y >= lb->y + 1 && mevent.y < LINES - 1) {
-               ListBox_setSelected(lb, mevent.y - lb->y + lb->scrollV - 1);
+            if (mevent.y >= panel->y + 1 && mevent.y < LINES - 1) {
+               Panel_setSelected(panel, mevent.y - panel->y + panel->scrollV - 1);
                doRefresh = false;
                refreshTimeout = resetRefreshTimeout;
                follow = true;
@@ -393,11 +438,8 @@ int main(int argc, char** argv) {
       }
 
       switch (ch) {
-      case ERR:
-         refreshTimeout--;
-         continue;
       case KEY_RESIZE:
-         ListBox_resize(lb, COLS, LINES-headerHeight-1);
+         Panel_resize(panel, COLS, LINES-headerHeight-1);
          if (incSearchMode)
             FunctionBar_draw(searchBar, incSearchBuffer);
          else
@@ -409,7 +451,7 @@ int main(int argc, char** argv) {
          pl->sortKey = PERCENT_MEM;
          pl->treeView = false;
          settings->changed = true;
-         ListBox_setRichHeader(lb, ProcessList_printHeader(pl));
+         Panel_setRichHeader(panel, ProcessList_printHeader(pl));
          break;
       }
       case 'T':
@@ -418,13 +460,13 @@ int main(int argc, char** argv) {
          pl->sortKey = TIME;
          pl->treeView = false;
          settings->changed = true;
-         ListBox_setRichHeader(lb, ProcessList_printHeader(pl));
+         Panel_setRichHeader(panel, ProcessList_printHeader(pl));
          break;
       }
       case 'U':
       {
-         for (int i = 0; i < ListBox_getSize(lb); i++) {
-            Process* p = (Process*) ListBox_get(lb, i);
+         for (int i = 0; i < Panel_getSize(panel); i++) {
+            Process* p = (Process*) Panel_get(panel, i);
             p->tag = false;
          }
          doRefresh = true;
@@ -436,13 +478,13 @@ int main(int argc, char** argv) {
          pl->sortKey = PERCENT_CPU;
          pl->treeView = false;
          settings->changed = true;
-         ListBox_setRichHeader(lb, ProcessList_printHeader(pl));
+         Panel_setRichHeader(panel, ProcessList_printHeader(pl));
          break;
       }
       case KEY_F(1):
       case 'h':
       {
-         showHelp();
+         showHelp(pl);
          FunctionBar_draw(defaultBar, NULL);
          refreshTimeout = 0;
          break;
@@ -456,14 +498,14 @@ int main(int argc, char** argv) {
       }
       case ' ':
       {
-         Process* p = (Process*) ListBox_getSelected(lb);
+         Process* p = (Process*) Panel_getSelected(panel);
          Process_toggleTag(p);
-         ListBox_onKey(lb, KEY_DOWN);
+         Panel_onKey(panel, KEY_DOWN);
          break;
       }
       case 's':
       {
-         TraceScreen* ts = TraceScreen_new((Process*) ListBox_getSelected(lb));
+         TraceScreen* ts = TraceScreen_new((Process*) Panel_getSelected(panel));
          TraceScreen_run(ts);
          TraceScreen_delete(ts);
          clear();
@@ -478,10 +520,10 @@ int main(int argc, char** argv) {
       {
          Setup_run(settings, headerHeight);
          // TODO: shouldn't need this, colors should be dynamic
-         ListBox_setRichHeader(lb, ProcessList_printHeader(pl));
+         Panel_setRichHeader(panel, ProcessList_printHeader(pl));
          headerHeight = Header_calculateHeight(header);
-         ListBox_move(lb, 0, headerHeight);
-         ListBox_resize(lb, COLS, LINES-headerHeight-1);
+         Panel_move(panel, 0, headerHeight);
+         Panel_resize(panel, COLS, LINES-headerHeight-1);
          FunctionBar_draw(defaultBar, NULL);
          refreshTimeout = 0;
          break;
@@ -493,14 +535,14 @@ int main(int argc, char** argv) {
       }
       case 'u':
       {
-         ListBox* lbu = ListBox_new(0, 0, 0, 0, LISTITEM_CLASS, true);
-         ListBox_setHeader(lbu, "Show processes of:");
-         UsersTable_foreach(ut, addUserToList, lbu);
-         TypedVector_sort(lbu->items);
+         Panel* usersPanel = Panel_new(0, 0, 0, 0, LISTITEM_CLASS, true, ListItem_compare);
+         Panel_setHeader(usersPanel, "Show processes of:");
+         UsersTable_foreach(ut, addUserToList, usersPanel);
+         Vector_sort(usersPanel->items);
          ListItem* allUsers = ListItem_new("All users", -1);
-         ListBox_insert(lbu, 0, (Object*) allUsers);
+         Panel_insert(usersPanel, 0, (Object*) allUsers);
          char* fuFunctions[2] = {"Show    ", "Cancel "};
-         ListItem* picked = (ListItem*) pickFromList(lb, lbu, 20, headerHeight, fuFunctions, defaultBar);
+         ListItem* picked = (ListItem*) pickFromList(panel, usersPanel, 20, headerHeight, fuFunctions, defaultBar);
          if (picked) {
             if (picked == allUsers) {
                userOnly = false;
@@ -514,34 +556,33 @@ int main(int argc, char** argv) {
       case KEY_F(9):
       case 'k':
       {
-         if (!lbk) {
-            lbk = (ListBox*) SignalsListBox_new(0, 0, 0, 0);
+         if (!killPanel) {
+            killPanel = (Panel*) SignalsPanel_new(0, 0, 0, 0);
          }
-         SignalsListBox_reset((SignalsListBox*) lbk);
+         SignalsPanel_reset((SignalsPanel*) killPanel);
          char* fuFunctions[2] = {"Send  ", "Cancel "};
-         Signal* signal = (Signal*) pickFromList(lb, lbk, 15, headerHeight, fuFunctions, defaultBar);
+         Signal* signal = (Signal*) pickFromList(panel, killPanel, 15, headerHeight, fuFunctions, defaultBar);
          if (signal) {
             if (signal->number != 0) {
-               ListBox_setHeader(lb, "Sending...");
-               ListBox_draw(lb, true);
+               Panel_setHeader(panel, "Sending...");
+               Panel_draw(panel, true);
                refresh();
                bool anyTagged = false;
-               for (int i = 0; i < ListBox_getSize(lb); i++) {
-                  Process* p = (Process*) ListBox_get(lb, i);
+               for (int i = 0; i < Panel_getSize(panel); i++) {
+                  Process* p = (Process*) Panel_get(panel, i);
                   if (p->tag) {
                      Process_sendSignal(p, signal->number);
-                     Process_toggleTag(p);
                      anyTagged = true;
                   }
                }
                if (!anyTagged) {
-                  Process* p = (Process*) ListBox_getSelected(lb);
+                  Process* p = (Process*) Panel_getSelected(panel);
                   Process_sendSignal(p, signal->number);
                }
                napms(500);
             }
          }
-         ListBox_setRichHeader(lb, ProcessList_printHeader(pl));
+         Panel_setRichHeader(panel, ProcessList_printHeader(pl));
          refreshTimeout = 0;
          break;
       }
@@ -556,25 +597,25 @@ int main(int argc, char** argv) {
       case '.':
       case KEY_F(6):
       {
-         ListBox* lbf = ListBox_new(0,0,0,0,LISTITEM_CLASS,true);
-         ListBox_setHeader(lbf, "Sort by");
+         Panel* sortPanel = Panel_new(0, 0, 0, 0, LISTITEM_CLASS, true, ListItem_compare);
+         Panel_setHeader(sortPanel, "Sort by");
          char* fuFunctions[2] = {"Sort  ", "Cancel "};
          ProcessField* fields = pl->fields;
          for (int i = 0; fields[i]; i++) {
             char* name = String_trim(Process_printField(fields[i]));
-            ListBox_add(lbf, (Object*) ListItem_new(name, fields[i]));
+            Panel_add(sortPanel, (Object*) ListItem_new(name, fields[i]));
             if (fields[i] == pl->sortKey)
-               ListBox_setSelected(lbf, i);
+               Panel_setSelected(sortPanel, i);
             free(name);
          }
-         ListItem* field = (ListItem*) pickFromList(lb, lbf, 15, headerHeight, fuFunctions, defaultBar);
+         ListItem* field = (ListItem*) pickFromList(panel, sortPanel, 15, headerHeight, fuFunctions, defaultBar);
          if (field) {
             pl->treeView = false;
             settings->changed = true;
             pl->sortKey = field->key;
          }
-         ((Object*)lbf)->delete((Object*)lbf);
-         ListBox_setRichHeader(lb, ProcessList_printHeader(pl));
+         ((Object*)sortPanel)->delete((Object*)sortPanel);
+         Panel_setRichHeader(panel, ProcessList_printHeader(pl));
          refreshTimeout = 0;
          break;
       }
@@ -591,14 +632,14 @@ int main(int argc, char** argv) {
       case '=':
       case '+':
       {
-         doRefresh = changePriority(lb, 1);
+         doRefresh = changePriority(panel, 1);
          break;
       }
       case KEY_F(7):
       case ']':
       case '-':
       {
-         doRefresh = changePriority(lb, -1);
+         doRefresh = changePriority(panel, -1);
          break;
       }
       case KEY_F(3):
@@ -625,7 +666,7 @@ int main(int argc, char** argv) {
       default:
          doRefresh = false;
          refreshTimeout = resetRefreshTimeout;
-         ListBox_onKey(lb, ch);
+         Panel_onKey(panel, ch);
          break;
       }
       follow = false;
@@ -642,9 +683,9 @@ int main(int argc, char** argv) {
    ProcessList_delete(pl);
    FunctionBar_delete((Object*)searchBar);
    FunctionBar_delete((Object*)defaultBar);
-   ((Object*)lb)->delete((Object*)lb);
-   if (lbk)
-      ((Object*)lbk)->delete((Object*)lbk);
+   ((Object*)panel)->delete((Object*)panel);
+   if (killPanel)
+      ((Object*)killPanel)->delete((Object*)killPanel);
    UsersTable_delete(ut);
    Settings_delete(settings);
    debug_done();
